Attribute VB_Name = "PingMod"

Option Explicit

Private Const IP_STATUS_BASE = 11000
Private Const IP_SUCCESS = 0
Private Const IP_BUF_TOO_SMALL = (11000 + 1)
Private Const IP_DEST_NET_UNREACHABLE = (11000 + 2)
Private Const IP_DEST_HOST_UNREACHABLE = (11000 + 3)
Private Const IP_DEST_PROT_UNREACHABLE = (11000 + 4)
Private Const IP_DEST_PORT_UNREACHABLE = (11000 + 5)
Private Const IP_NO_RESOURCES = (11000 + 6)
Private Const IP_BAD_OPTION = (11000 + 7)
Private Const IP_HW_ERROR = (11000 + 8)
Private Const IP_PACKET_TOO_BIG = (11000 + 9)
Private Const IP_REQ_TIMED_OUT = (11000 + 10)
Private Const IP_BAD_REQ = (11000 + 11)
Private Const IP_BAD_ROUTE = (11000 + 12)
Private Const IP_TTL_EXPIRED_TRANSIT = (11000 + 13)
Private Const IP_TTL_EXPIRED_REASSEM = (11000 + 14)
Private Const IP_PARAM_PROBLEM = (11000 + 15)
Private Const IP_SOURCE_QUENCH = (11000 + 16)
Private Const IP_OPTION_TOO_BIG = (11000 + 17)
Private Const IP_BAD_DESTINATION = (11000 + 18)
Private Const IP_ADDR_DELETED = (11000 + 19)
Private Const IP_SPEC_MTU_CHANGE = (11000 + 20)
Private Const IP_MTU_CHANGE = (11000 + 21)
Private Const IP_UNLOAD = (11000 + 22)
Private Const IP_ADDR_ADDED = (11000 + 23)
Private Const IP_GENERAL_FAILURE = (11000 + 50)
Private Const MAX_IP_STATUS = 11000 + 50
Private Const IP_PENDING = (11000 + 255)
Private Const PING_TIMEOUT = 1000
Private Const WS_VERSION_REQD = &H101
Private Const WS_VERSION_MAJOR = WS_VERSION_REQD \ &H100 And &HFF&
Private Const WS_VERSION_MINOR = WS_VERSION_REQD And &HFF&
Private Const MIN_SOCKETS_REQD = 1
Private Const SOCKET_ERROR = -1
Private Const MAX_WSADescription = 256
Private Const MAX_WSASYSStatus = 128

Private Type ICMP_OPTIONS
    Ttl As Byte
    Tos As Byte
    Flags As Byte
    OptionsSize As Byte
    OptionsData As Long
End Type

Private Type ICMP_ECHO_REPLY
    Address As Long
    Status As Long
    RoundTripTime As Long
    DataSize As Integer
    Reserved As Integer
    DataPointer As Long
    Options As ICMP_OPTIONS
    Data As String * 250
End Type

Private Type WSADATA
    wVersion As Integer
    wHighVersion As Integer
    szDescription(0 To MAX_WSADescription) As Byte
    szSystemStatus(0 To MAX_WSASYSStatus) As Byte
    wMaxSockets As Integer
    wMaxUDPDG As Integer
    dwVendorInfo As Long
End Type

Private Declare Function IcmpCloseHandle Lib "icmp" (ByVal IcmpHandle As Long) As Long
Private Declare Function IcmpCreateFile Lib "icmp" () As Long
Private Declare Function IcmpSendEcho Lib "icmp" (ByVal IcmpHandle As Long, ByVal DestinationAddress As Long, ByVal RequestData As String, ByVal RequestSize As Integer, ByVal RequestOptions As Long, ReplyBuffer As ICMP_ECHO_REPLY, ByVal ReplySize As Long, ByVal Timeout As Long) As Long
Private Declare Function WSAStartup Lib "wsock32" (ByVal wVersionRequired As Long, lpWSADATA As WSADATA) As Long
Private Declare Function WSACleanup Lib "wsock32" () As Long


Private Function GetStatusCode(Status As Long) As String
Dim msg As String
    Select Case Status
        Case IP_SUCCESS: msg = "Success"
        Case IP_BUF_TOO_SMALL: msg = "Buffer too small"
        Case IP_DEST_NET_UNREACHABLE: msg = "Dest. net unreachable"
        Case IP_DEST_HOST_UNREACHABLE: msg = "Dest. host unreachable"
        Case IP_DEST_PROT_UNREACHABLE: msg = "Dest. prot unreachable"
        Case IP_DEST_PORT_UNREACHABLE: msg = "Dest. port unreachable"
        Case IP_NO_RESOURCES: msg = "No resources"
        Case IP_BAD_OPTION: msg = "Bad option"
        Case IP_HW_ERROR: msg = "Hardware Error"
        Case IP_PACKET_TOO_BIG: msg = "Packet too big"
        Case IP_REQ_TIMED_OUT: msg = "Request timed out"
        Case IP_BAD_REQ: msg = "Bad request"
        Case IP_BAD_ROUTE: msg = "Bad route"
        Case IP_TTL_EXPIRED_TRANSIT: msg = "TTL Expired Transit"
        Case IP_TTL_EXPIRED_REASSEM: msg = "TTL Expired Reassem"
        Case IP_PARAM_PROBLEM: msg = "Param problem"
        Case IP_SOURCE_QUENCH: msg = "Source quench"
        Case IP_OPTION_TOO_BIG: msg = "Option too big"
        Case IP_BAD_DESTINATION: msg = "Bad destination"
        Case IP_ADDR_DELETED: msg = "Address deleted"
        Case IP_SPEC_MTU_CHANGE: msg = "Spec. MTU Change"
        Case IP_MTU_CHANGE: msg = "MTU change"
        Case IP_UNLOAD: msg = "Unload"
        Case IP_ADDR_ADDED: msg = "Address added"
        Case IP_GENERAL_FAILURE: msg = "General failure"
        Case IP_PENDING: msg = "Pending..."
        Case PING_TIMEOUT: msg = "Ping timeout"
        Case Else: msg = "Unknown"
    End Select
    GetStatusCode = msg
End Function



Private Function HiByte(ByVal wParam As Integer)
    HiByte = wParam \ &H100 And &HFF&
End Function



Private Function LoByte(ByVal wParam As Integer)
    LoByte = wParam And &HFF&
End Function



Public Function PingIP(szAddress As String, Optional intStatusCode As Integer, Optional strStatusMsg As String) As Long
Dim hPort As Long
Dim dwAddress As Long
Dim iOpt As Long
Dim ECHO As ICMP_ECHO_REPLY
Const sDataToSend As String = "Echo This"
    dwAddress = AddressStringToLong(szAddress)
    If Not SocketsInitialize Then
        PingIP = 0
        intStatusCode = -1
        strStatusMsg = "Unable to initialize sockets."
        Exit Function
    End If
    hPort = IcmpCreateFile()
    IcmpSendEcho hPort, dwAddress, sDataToSend, Len(sDataToSend), 0, ECHO, Len(ECHO), PING_TIMEOUT
    PingIP = ECHO.RoundTripTime
    intStatusCode = ECHO.Status
    strStatusMsg = GetStatusCode(ECHO.Status)
    IcmpCloseHandle hPort
    SocketsCleanup
End Function



Private Function AddressStringToLong(ByVal tmp As String) As Long
Dim cntr As Integer
Dim parts(1 To 4) As String
Dim Inst As Integer
    cntr = 0
    Inst = InStr(tmp, ".")
    While Inst > 0
        cntr = cntr + 1
        parts(cntr) = Mid(tmp, 1, InStr(tmp, ".") - 1)
        tmp = Mid(tmp, InStr(tmp, ".") + 1)
        Inst = InStr(tmp, ".")
        If cntr = 3 Then Inst = 0
    Wend
    cntr = cntr + 1
    parts(cntr) = tmp
    If cntr <> 4 Then
        AddressStringToLong = 0
        Exit Function
    End If
    AddressStringToLong = Val("&H" & Right("00" & Hex(parts(4)), 2) & _
    Right("00" & Hex(parts(3)), 2) & _
    Right("00" & Hex(parts(2)), 2) & _
    Right("00" & Hex(parts(1)), 2))
End Function



Private Function SocketsCleanup() As Boolean
Dim X As Long
    X = WSACleanup()
    If X <> 0 Then
        MsgBox "Windows Sockets Error " & X & " occurred in Cleanup.", vbExclamation
        SocketsCleanup = False
    Else
        SocketsCleanup = True
    End If
End Function



Private Function SocketsInitialize() As Boolean
Dim WSAD As WSADATA
Dim X As Integer
Dim szLoByte As String, szHiByte As String, szBuf As String
    X = WSAStartup(WS_VERSION_REQD, WSAD)
    If X <> 0 Then
        MsgBox "Windows Sockets For 32 bit Windows environments is not successfully responding.", vbCritical
        SocketsInitialize = False
        Exit Function
    End If
    If LoByte(WSAD.wVersion) < WS_VERSION_MAJOR Or _
    (LoByte(WSAD.wVersion) = WS_VERSION_MAJOR And _
    HiByte(WSAD.wVersion) < WS_VERSION_MINOR) Then
        szHiByte = Trim$(Str$(HiByte(WSAD.wVersion)))
        szLoByte = Trim$(Str$(LoByte(WSAD.wVersion)))
        szBuf = "Windows Sockets Version " & szLoByte & "." & szHiByte
        szBuf = szBuf & " is not supported by Windows " & _
        "Sockets For 32 bit Windows environments."
        MsgBox szBuf, vbExclamation
        SocketsInitialize = False
        Exit Function
    End If
    If WSAD.wMaxSockets < MIN_SOCKETS_REQD Then
        szBuf = "This application requires a minimum of " & _
        CStr(MIN_SOCKETS_REQD) & " supported sockets."
        MsgBox szBuf, vbExclamation
        SocketsInitialize = False
        Exit Function
    End If
    SocketsInitialize = True
End Function
